﻿# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# Function: Log informational message.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function Log([string]$msg)
{
    $now = [datetime]::Now.ToString("HH:mm:ss")
    Write-Host " ", $now, " - ", $msg
} 

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# Function: Log error message.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
$hasErrors = $false
function LogError([string]$msg)
{
    $now = [datetime]::Now.ToString("HH:mm:ss")
    Write-Host -Fore Red " ", $now, " - ", $msg
    $hasErrors = $true
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# Function: Download Federation Metadata
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function GetFederationMetadata([string]$url)
{
	Log "Downloading Federation Metadata from $url."
    
	$webclient = New-Object System.Net.WebClient
	$data = $webclient.DownloadData($url);
	$ms = new-object io.memorystream(,$data);
	$ms.Flush();
	$ms.Position = 0;
	$fedMetadata = new-object XML
	$fedMetadata.Load($ms)
	Return $fedMetadata
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# Function: Get Certificate.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function GetCertificate([xml]$fedMetadata) {
	$ns = @{xsi = 'http://www.w3.org/2001/XMLSchema-instance'}
	$roleDescriptor = Select-Xml "//*[@xsi:type[contains(.,'SecurityTokenServiceType')]]" $fedMetadata -Namespace $ns
	if (-Not $roleDescriptor) {
		LogError "The <RoleDescriptor> element with xsi:type='fed:SecurityTokenServiceType' was not found";
	}

	Return $roleDescriptor.Node.KeyDescriptor.KeyInfo.X509Data.X509Certificate
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# Function: Get Tenant.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function GetTenant([xml]$fedMetadata) {
	$ns = @{xsi = 'http://www.w3.org/2001/XMLSchema-instance'}
	$roleDescriptor = Select-Xml "//*[@xsi:type[contains(.,'SecurityTokenServiceType')]]" $fedMetadata -Namespace $ns
	if (-Not $roleDescriptor) {
		LogError "The <RoleDescriptor> element with xsi:type='fed:SecurityTokenServiceType' was not found";
	}

	Return $roleDescriptor.Node.Organization.OrganizationName.InnerText
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# Function: Get SignOnUrl.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function GetSignInUrl([xml]$fedMetadata) {
	$ns = @{xsi = 'http://www.w3.org/2001/XMLSchema-instance'}
	$roleDescriptor = Select-Xml "//*[@xsi:type[contains(.,'SecurityTokenServiceType')]]" $fedMetadata -Namespace $ns
	if (-Not $roleDescriptor) {
		LogError "The <RoleDescriptor> element with xsi:type='fed:SecurityTokenServiceType' was not found";
	}

	Return $roleDescriptor.Node.SecurityTokenServiceEndpoint.EndpointReference.Address
}

function Add-TihTrustedLoginProvider{
    [CmdletBinding()]
	Param
	(
        [Parameter(Mandatory=$true)]
        [string]$Realm,
		[string]$FedMetadataUrl,
        [string]$Name = '',
        [bool]$UseEmailAddressAsUniqueKey = $false
	)

    $claimProviderName = "TIHCP"
    $certName = "tih.cer"
    
    # Loading the metadata
    $fedMetadata = $null
    If (Test-Path("FederationMetadata.xml")) {
    	Log "Using local FederationMetadata file."
    	$fedMetadata = Get-Content FederationMetadata.xml	
    }
    Else {
    	Log "Downloading from: $FedMetadataUrl"
        $fedMetadata = GetFederationMetadata($FedMetadataUrl)
    }

    if($fedMetadata -eq $null) {
        LogError "No metadata was found"
        Return
    }

    # Get WS-Federation parameters
    $tenant = GetTenant($fedMetadata)
    $signInUrl = GetSignInUrl($fedMetadata)

    Log "Found realm '$realm'."
    Log "Found tenant '$tenant'."
    Log "Found signInUrl '$signInUrl'."
	
    # Get signing certificate
	GetCertificate($fedMetadata) | Set-Content $certName
    $signingCert = Get-PfxCertificate $certName

    Log "Found signingCertificate with thumbprint '$($signingCert.Thumbprint)'."

    $tempCert = Get-PfxCertificate $certName
	$certs = @()
	$certs += $tempCert

	Log "Certificates: $certs"

    # Construct the claim mappings
    $identifierClaimType = "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name";
    $roleClaimType = "http://schemas.microsoft.com/ws/2008/06/identity/claims/role";
    $sharePointIdentifierClaimType = "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/upn";
    $emailAddressClaimType = "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress";

    $upnCtm = New-SPClaimTypeMapping -IncomingClaimTypeDisplayName "UPN" -IncomingClaimType $identifierClaimType -LocalClaimType $sharePointIdentifierClaimType
    $roleCtm = New-SPClaimTypeMapping -IncomingClaimTypeDisplayName "Role" -IncomingClaimType $roleClaimType -SameAsIncoming
    $emailCtm = New-SPClaimTypeMapping -IncomingClaimTypeDisplayName "EmailAddress" -IncomingClaimType $emailAddressClaimType -SameAsIncoming

    $claimMappings = @()
    $claimMappings += $upnCtm
    $claimMappings += $roleCtm

    if($UseEmailAddressAsUniqueKey -eq $true){
        $claimMappings += $emailCtm
    }

    # Get the identifying claimtype
    $identifierClaimType;
    
    if($UseEmailAddressAsUniqueKey -eq $true){
        $identifierClaimType = $emailAddressClaimType
    }
    else{
        $identifierClaimType = $identifierClaimType
    }

    # Add Tih as Trusted Identity Token Issuer
    $providerName = ''
    if($Name.Length -eq 0) {
        $providerName = 'The Identity Hub ' + $tenant
    }
    else {
        $providerName = $Name
    }
        
    $spti = Get-SPTrustedIdentityTokenIssuer | ?{ $_.Name -eq $providerName }

    if($spti -eq $null){
        Log "Creating new SPTrustedIdentityTokenIssuer: '$providerName'."

        $spti = New-SPTrustedIdentityTokenIssuer -Name $providerName -Description $providerName -Realm $Realm -ImportTrustCertificate $signingCert -SignInUrl $signInUrl -ClaimsMappings $claimMappings -IdentifierClaim $identifierClaimType -UseWReply
    }
    else{
        Log "SPTrustedIdentityTokenIssuer '$providerName' already exists."

        $previouslyInexistentMappings = @()

		foreach ($claimMapping in $mappings) {
			$isRepeated = $false
			foreach ($claimTypeInformation in $spti.ClaimTypeInformation) {   
				if ($claimMapping.DisplayName -eq $claimTypeInformation.DisplayName) {
					if ($claimMapping.InputClaimType -ne $claimTypeInformation.InputClaimType) {      
						Log " ! ClaimType '$($claimTypeInformation.DisplayName)' already in use."
					}
					$isRepeated = $true
				}
				else {
					if ($claimMapping.InputClaimType -eq $claimTypeInformation.InputClaimType) {
						Log " ! ClaimType '$($claimTypeInformation.DisplayName)' already in use."
					}           
				}
			}
			if ($isRepeated -ne $true) {
				$previouslyInexistentMappings += $claimMapping
			}
		}

        Log "Adding claims..."
		foreach ($claimMapping in $previouslyInexistentMappings) {
			Log " > Adding ClaimType $claimMapping.InputClaimType"
			$spti.ClaimTypes.Add($claimMapping.InputClaimType)
			$spti.Update()
			
			Add-SPClaimTypeMapping -Identity $claimMapping -TrustedIdentityTokenIssuer $spti
			Log "   Added claim mapping: '$($claimMapping.DisplayName)' '$($claimMapping.InputClaimType)'."
		}

        Log "Updating signing certificate"
        $spti.SigningCertificate = $signingCert
        $spti.ProviderUri = $signInUrl
        $spti.Update()
    }

    # Add claim provider to the identity token issuer
    Log "Setting claim provider '$claimProviderName' on trusted login provider."

    $claimProvider = Get-SPClaimProvider | ?{ $_.ClaimProvider.Name -eq $claimProviderName }

    if ($claimProvider -eq $null) {
        LogError "Claim provider '$claimProviderName' does not exist in the farm."
    }

    if ($spti.ClaimProviderName -ne $claimProviderName) {
        $spti.ClaimProviderName = $claimProviderName
        $spti.Update()
    }

    # Add STS certificate and its certificate chain as trusted.
	$existingTrustedRootAuth = Get-SPTrustedRootAuthority
	foreach ($tempCert in $certs) {
		$certName = ([regex]'CN=([^,]+)').matches($tempCert.Subject) | foreach {$_.Groups[1].Value}
		
		Log "Checking if certificate $certName exists in SP trusted root."  
		$trustedRootAuthExists = $false
		
		foreach ($trustedRootAuth in $existingTrustedRootAuth) {
			if ($trustedRootAuth.Name -Eq $certName) {
				Log " > Certificate $certName exists in SP trusted root."  
				$trustedRootAuthExists = $true
				break
			}
		}
		
		if ($trustedRootAuthExists -Ne $true) {
			Log "Certificate $certName does not exist in SP trusted root. Adding certificate $certName to SP trusted root."
			New-SPTrustedRootAuthority -name $certName -Certificate $tempCert
		}
	}
}